{==============================================================================}
{                                                                              }
{       Borland Delphi Runtime Library                                         }
{       GLX 1.4 - Headertranslation                                            }
{         OpenGl Extension for the X Window System (XFree86 & Co.)             }
{       Version 0.1a                                                           }
{       Date : 11.02.2004                                                      }
{                                                                              }
{==============================================================================}
{                                                                              }
{       Containts the translations of glx.h, glxtokens.h                       }
{       It also contains some helperfunctions that were inspired by those      }
{       found in Mike Lischke's OpenGL12.pas.                                  }
{                                                                              }
{       Copyright (C) DGL-OpenGL2-Portteam                                     }
{       All Rights Reserved                                                    }
{                                                                              }
{       Obtained through:                                                      }
{       Delphi OpenGL Community(DGL) - www.delphigl.com                        }
{                                                                              }
{       Converted and maintained by DGL's GL2.0-Team :                         }
{         - Sascha Willems (Son of Satan) - http://www.delphigl.de             }
{         - Steffen Xonna (LossyEx)       - http://www.dev-center.de           }
{         - Lars Middendorf               - http://www.3d-seite.de             }
{         - Nico Michaelis                - http://www.delphigl.com/           }
{       Additional input :                                                     }
{         - Martin Waldegger (Mars)       - http://www.basegraph.com           }
{                                                                              }
{==============================================================================}
{      If you have problems on using our unit, then please take a look at      }
{      the "how to use.txt".                                                   }
{==============================================================================}
{ You may retrieve the latest version of this file at the Delphi OpenGL        }
{ Community home page, located at http://www.delphigl.com/                     }
{                                                                              }
{ The contents of this file are used with permission, subject to               }
{ the Mozilla Public License Version 1.1 (the "License"); you may              }
{ not use this file except in compliance with the License. You may             }
{ obtain a copy of the License at                                              }
{ http://www.mozilla.org/MPL/MPL-1.1.html                                      }
{                                                                              }
{ Software distributed under the License is distributed on an                  }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or               }
{ implied. See the License for the specific language governing                 }
{ rights and limitations under the License.                                    }
{                                                                              }
{==============================================================================}
{ History :                                                                    }
{ Version 0.1a - Beta Testing Release                                     (NM) }
{==============================================================================}
unit dglGLX;

interface

//This is a header for X-Window System. Do not try to use this file for
//Win32! This is senseless!
uses
  SysUtils, Classes, xlib, dglOpenGL;

const
  ShowFunctions = false; //Zeigt auf StdOut an, ob eine funktion untersttzt
                         //wird, oder nicht

{ XFree86: glxtokens.h v 1.5 2001/03/21}
const
{** Visual Config Attributes (glXGetConfig, glXGetFBConfigAttrib)}
  GLX_USE_GL            = 1;	{  support GLX rendering  }
  GLX_BUFFER_SIZE       = 2;	{  depth of the color buffer  }
  GLX_LEVEL             = 3;	{  level in plane stacking  }
  GLX_RGBA              = 4;	{  true if RGBA mode  }
  GLX_DOUBLEBUFFER      = 5;	{  double buffering supported  }
  GLX_STEREO            = 6;	{  stereo buffering supported  }
  GLX_AUX_BUFFERS       = 7;	{  number of aux buffers  }
  GLX_RED_SIZE          = 8;	{  number of red component bits  }
  GLX_GREEN_SIZE        = 9;	{  number of green component bits  }
  GLX_BLUE_SIZE         = 10;	{  number of blue component bits  }
  GLX_ALPHA_SIZE        = 11;	{  number of alpha component bits  }
  GLX_DEPTH_SIZE        = 12;	{  number of depth bits  }
  GLX_STENCIL_SIZE      = 13;	{  number of stencil bits  }
  GLX_ACCUM_RED_SIZE    = 14;	{  number of red accum bits  }
  GLX_ACCUM_GREEN_SIZE  = 15;	{  number of green accum bits  }
  GLX_ACCUM_BLUE_SIZE   = 16;	{  number of blue accum bits  }
  GLX_ACCUM_ALPHA_SIZE  = 17;	{  number of alpha accum bits  }

{** FBConfig-specific attributes}
  GLX_X_VISUAL_TYPE             = $22;
  GLX_CONFIG_CAVEAT             = $20;	{  Like visual_info VISUAL_CAVEAT_EXT  }
  GLX_TRANSPARENT_TYPE          = $23;
  GLX_TRANSPARENT_INDEX_VALUE   = $24;
  GLX_TRANSPARENT_RED_VALUE     = $25;
  GLX_TRANSPARENT_GREEN_VALUE   = $26;
  GLX_TRANSPARENT_BLUE_VALUE    = $27;
  GLX_TRANSPARENT_ALPHA_VALUE   = $28;
  GLX_DRAWABLE_TYPE             = $8010;
  GLX_RENDER_TYPE               = $8011;
  GLX_X_RENDERABLE              = $8012;
  GLX_FBCONFIG_ID               = $8013;
  GLX_MAX_PBUFFER_WIDTH         = $8016;
  GLX_MAX_PBUFFER_HEIGHT        = $8017;
  GLX_MAX_PBUFFER_PIXELS        = $8018;
  GLX_VISUAL_ID                 = $800B;

{** Error return values from glXGetConfig.  Success is indicated by
** a value of 0. }
  GLX_BAD_SCREEN                = 1;	{  screen # is bad  }
  GLX_BAD_ATTRIBUTE             = 2;	{  attribute to get is bad  }
  GLX_NO_EXTENSION              = 3;	{  no glx extension on server  }
  GLX_BAD_VISUAL                = 4;	{  visual # not known by GLX  }
  GLX_BAD_CONTEXT               = 5;	{  returned only by import_context EXT?  }
  GLX_BAD_VALUE                 = 6;	{  returned only by glXSwapIntervalSGI?  }
  GLX_BAD_ENUM                  = 7;	{  unused?  }

{  FBConfig attribute values  }

{** Generic "don't care" value for glX ChooseFBConfig attributes (except
** GLX_LEVEL)}
  GLX_DONT_CARE                 = $FFFFFFFF;

{  GLX_RENDER_TYPE bits  }
  GLX_RGBA_BIT                  = $00000001;
  GLX_COLOR_INDEX_BIT           = $00000002;

{  GLX_DRAWABLE_TYPE bits  }
  GLX_WINDOW_BIT                = $00000001;
  GLX_PIXMAP_BIT                = $00000002;
  GLX_PBUFFER_BIT               = $00000004;

{  GLX_CONFIG_CAVEAT attribute values  }
  GLX_NONE                      = $8000;
  GLX_SLOW_CONFIG               = $8001;
  GLX_NON_CONFORMANT_CONFIG     = $800D;

{  GLX_X_VISUAL_TYPE attribute values  }
  GLX_TRUE_COLOR                = $8002;
  GLX_DIRECT_COLOR              = $8003;
  GLX_PSEUDO_COLOR              = $8004;
  GLX_STATIC_COLOR              = $8005;
  GLX_GRAY_SCALE                = $8006;
  GLX_STATIC_GRAY               = $8007;

{  GLX_TRANSPARENT_TYPE attribute values  }
{    GLX_NONE = $8000;  }
  GLX_TRANSPARENT_RGB           = $8008;
  GLX_TRANSPARENT_INDEX         = $8009;

{  glXCreateGLXPbuffer attributes  }
  GLX_PRESERVED_CONTENTS        = $801B;
  GLX_LARGEST_PBUFFER           = $801C;
  GLX_PBUFFER_HEIGHT            = $8040; {  New for GLX 1.3  }
  GLX_PBUFFER_WIDTH             = $8041; {  New for GLX 1.3  }

{  glXQueryGLXPBuffer attributes  }
  GLX_WIDTH                     = $801D;
  GLX_HEIGHT                    = $801E;
  GLX_EVENT_MASK                = $801F;

{  glXCreateNewContext render_type attribute values  }
  GLX_RGBA_TYPE                 = $8014;
  GLX_COLOR_INDEX_TYPE          = $8015;

{  glXQueryContext attributes  }
{    GLX_FBCONFIG_ID            = $8013;  }
{    GLX_RENDER_TYPE            = $8011;  }
  GLX_SCREEN                    = $800C;

{  glXSelectEvent event mask bits  }
  GLX_PBUFFER_CLOBBER_MASK      = $08000000;

{  GLXPbufferClobberEvent event_type values  }
  GLX_DAMAGED                   = $8020;
  GLX_SAVED                     = $8021;

{  GLXPbufferClobberEvent draw_type values  }
  GLX_WINDOW                    = $8022;
  GLX_PBUFFER                   = $8023;

{  GLXPbufferClobberEvent buffer_mask bits  }
  GLX_FRONT_LEFT_BUFFER_BIT     = $00000001;
  GLX_FRONT_RIGHT_BUFFER_BIT    = $00000002;
  GLX_BACK_LEFT_BUFFER_BIT      = $00000004;
  GLX_BACK_RIGHT_BUFFER_BIT     = $00000008;
  GLX_AUX_BUFFERS_BIT           = $00000010;
  GLX_DEPTH_BUFFER_BIT          = $00000020;
  GLX_STENCIL_BUFFER_BIT        = $00000040;
  GLX_ACCUM_BUFFER_BIT          = $00000080;

{** Extension return values from glXGetConfig.  These are also
** accepted as parameter values for glXChooseVisual.}

  GLX_X_VISUAL_TYPE_EXT           = $22;	{  visual_info extension type  }
  GLX_TRANSPARENT_TYPE_EXT        = $23;	{  visual_info extension  }
  GLX_TRANSPARENT_INDEX_VALUE_EXT = $24;	{  visual_info extension  }
  GLX_TRANSPARENT_RED_VALUE_EXT   = $25;	{  visual_info extension  }
  GLX_TRANSPARENT_GREEN_VALUE_EXT = $26;	{  visual_info extension  }
  GLX_TRANSPARENT_BLUE_VALUE_EXT  = $27;	{  visual_info extension  }
  GLX_TRANSPARENT_ALPHA_VALUE_EXT = $28;	{  visual_info extension  }

{  Property values for visual_type  }
  GLX_TRUE_COLOR_EXT              = $8002;
  GLX_DIRECT_COLOR_EXT            = $8003;
  GLX_PSEUDO_COLOR_EXT            = $8004;
  GLX_STATIC_COLOR_EXT            = $8005;
  GLX_GRAY_SCALE_EXT              = $8006;
  GLX_STATIC_GRAY_EXT             = $8007;

{  Property values for transparent pixel  }
  GLX_NONE_EXT                    = $8000;
  GLX_TRANSPARENT_RGB_EXT         = $8008;
  GLX_TRANSPARENT_INDEX_EXT       = $8009;

{  Property values for visual_rating  }
  GLX_VISUAL_CAVEAT_EXT                 = $20;  {  visual_rating extension type  }
  GLX_SLOW_VISUAL_EXT                   = $8001;
  GLX_NON_CONFORMANT_VISUAL_EXT         = $800D;

{** Names for attributes to glXGetClientString.}
  GLX_VENDOR                            = $1;
  GLX_VERSION                           = $2;
  GLX_EXTENSIONS                        = $3;

{** Names for attributes to glXQueryContextInfoEXT.}
  GLX_SHARE_CONTEXT_EXT                 = $800A;	{  id of share context  }
  GLX_VISUAL_ID_EXT                     = $800B;	{  id of context's visual  }
  GLX_SCREEN_EXT                        = $800C;	{  screen number  }

{* GLX 1.4 and later:}
  GLX_SAMPLE_BUFFERS_SGIS               = 100000;
  GLX_SAMPLES_SGIS                      = 100001;

{ XFree86: glx.h 1.9 2002/02/22}

{** GLX resources.}
type
  GLXContextID = XID;
  GLXPixmap = XID;
  GLXDrawable = XID;
  GLXPbuffer = XID;
  GLXWindow = XID;
  GLXFBConfigID = XID;

{** GLXContext is a pointer to opaque data.}
  GLXContext      = Pointer;

{** GLXFBConfig is a pointer to opaque data.}
  GLXFBConfig     = Pointer;

{ ********************************************************************** }

var
  glXChooseVisual : function (dpy : PDisplay; screen : glint; attribList : pglint): PXVisualInfo; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCopyContext : procedure (dpy : PDisplay; src : GLXContext; dst : GLXContext; mask : gluint); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCreateContext : function (dpy : PDisplay; vis : PXVisualInfo; shareList : GLXContext; direct : glboolean): GLXContext; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCreateGLXPixmap : function (dpy : PDisplay; vis : PXVisualInfo; pixmap : Pixmap): GLXPixmap {$IFDEF LINUX} cdecl; {$ENDIF}
  glXDestroyContext : procedure (dpy : PDisplay; ctx : GLXContext); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXDestroyGLXPixmap : procedure (dpy : PDisplay; pix : GLXPixmap); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetConfig : function (dpy : PDisplay; vis : PXVisualInfo; attrib : glint; value : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurrentContext : function : GLXContext {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurrentDrawable : function : GLXDrawable {$IFDEF LINUX} cdecl; {$ENDIF}
  glXIsDirect : function (dpy : PDisplay; ctx : GLXContext): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXMakeCurrent : function (dpy : PDisplay; drawable : GLXDrawable; ctx : GLXContext): glboolean {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryExtension : function (dpy : PDisplay; errorBase : pglint; eventBase : pglint): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryVersion : function (dpy : PDisplay; major : pglint; minor : pglint): glboolean {$IFDEF LINUX} cdecl; {$ENDIF}
  glXSwapBuffers : procedure (dpy : PDisplay; drawable : GLXDrawable); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXUseXFont : procedure (font : Font; first : glint; count : glint; listBase : glint);  {$IFDEF LINUX} cdecl; {$ENDIF}
  glXWaitGL : procedure  {$IFDEF LINUX} cdecl; {$ENDIF}
  glXWaitX : procedure {$IFDEF LINUX} cdecl; {$ENDIF}

  glXGetClientString : function (dpy : PDisplay; name : glint): PChar; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryServerString : function (dpy : PDisplay; screen : glint; name : glint): PChar; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryExtensionsString : function (dpy : PDisplay; screen : glint): PChar; {$IFDEF LINUX} cdecl; {$ENDIF}

{  New for GLX 1.3  }
  glXGetFBConfigs : function (dpy : PDisplay; screen : glint; nelements : pglint): GLXFBConfig; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXChooseFBConfig : function (dpy : PDisplay; screen : glint; attrib_list : pglint; nelements : pglint): GLXFBConfig; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetFBConfigAttrib : function (dpy : PDisplay; config : GLXFBConfig; attribute : glint; value : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetVisualFromFBConfig : function (dpy : PDisplay; config : GLXFBConfig) : PXVisualInfo;
  glXCreateWindow : function (dpy : PDisplay; config : GLXFBConfig; win : Window; attrib_list : pglint): GLXWindow; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXDestroyWindow : procedure (dpy : PDisplay; win : GLXWindow); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCreatePixmap : function (dpy : PDisplay; config : GLXFBConfig; pixmap : Pixmap; attrib_list : pglint): GLXPixmap; {$IFDEF LINUX} cdecl; {$ENDIF}

  glXDestroyPixmap : procedure (dpy : PDisplay; pixmap : GLXPixmap); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCreatePbuffer : function (dpy : PDisplay; config : GLXFBConfig; attrib_list : pglint): GLXPbuffer; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXDestroyPbuffer : procedure (dpy : PDisplay; pbuf : GLXPbuffer); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryDrawable : procedure (dpy : PDisplay; draw : GLXDrawable; attribute : glint; value : pgluint); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXCreateNewContext : function (dpy : PDisplay; config : GLXFBConfig; render_type : glint; share_list : GLXContext; direct : glboolean): GLXContext {$IFDEF LINUX} cdecl; {$ENDIF}
  glXMakeContextCurrent : function (display : PDisplay; draw : GLXDrawable; read : GLXDrawable; ctx : GLXContext): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurrentReadDrawable : function : GLXDrawable; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurreentDisplay : function : PDisplay;

  glXQueryContext : function (dpy : PDisplay; ctx : GLXContext; attribute : glint; value : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXSelectEvent : procedure (dpy : PDisplay; draw : GLXDrawable; event_mask : gluint); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetSelectedEvent : procedure (dpy : PDisplay; draw : GLXDrawable; event_mask : pgluint); {$IFDEF LINUX} cdecl; {$ENDIF}

{ GLX 1.4 and later  }
  //extern void (*glXGetProcAddress(const GLubyte *procname))(void);
  glXGetProcAddress : function (procname : PChar): Pointer; {$IFDEF LINUX} cdecl; {$ENDIF}

{  GLX extensions  }
  glXGetContextIDEXT : function (ctx : GLXContext): GLXContextID; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXImportContextEXT : function (dpy : PDisplay; contextID : GLXContextID): GLXContext; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXFreeContextEXT : procedure (dpy : PDisplay; ctx : GLXContext); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryContextInfoEXT : function (dpy : PDisplay; ctx : GLXContext; attribute : glint; value : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurrentDisplayEXT : function : PDisplay; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetProcAddressARB : function(procName : PChar): Pointer; {$IFDEF LINUX} cdecl; {$ENDIF}

{ XFree86(?): glxext.h 2002/03/22}

const
  GLX_GLXEXT_VERSION                    = 5;

  // GLX 1.4
  GLX_SAMPLE_BUFFERS                    = 100000;
  GLX_SAMPLES                           = 100001;

  //GLX_ARB_multisample
  GLX_SAMPLE_BUFFERS_ARB                = 100000;
  GLX_SAMPLES_ARB                       = 100001;

  //GLX_SGIX_fbconfig
  GLX_WINDOW_BIT_SGIX                   = $00000001;
  GLX_PIXMAP_BIT_SGIX                   = $00000002;
  GLX_RGBA_BIT_SGIX                     = $00000001;
  GLX_COLOR_INDEX_BIT_SGIX              = $00000002;
  GLX_DRAWABLE_TYPE_SGIX                = $8010;
  GLX_RENDER_TYPE_SGIX                  = $8011;
  GLX_X_RENDERABLE_SGIX                 = $8012;
  GLX_FBCONFIG_ID_SGIX                  = $8013;
  GLX_RGBA_TYPE_SGIX                    = $8014;
  GLX_COLOR_INDEX_TYPE_SGIX             = $8015;

  //GLX_SGIX_pbuffer
  GLX_PBUFFER_BIT_SGIX                  = $00000004;
  GLX_BUFFER_CLOBBER_MASK_SGIX          = $08000000;
  GLX_FRONT_LEFT_BUFFER_BIT_SGIX        = $00000001;
  GLX_FRONT_RIGHT_BUFFER_BIT_SGIX       = $00000002;
  GLX_BACK_LEFT_BUFFER_BIT_SGIX         = $00000004;
  GLX_BACK_RIGHT_BUFFER_BIT_SGIX        = $00000008;
  GLX_AUX_BUFFERS_BIT_SGIX              = $00000010;
  GLX_DEPTH_BUFFER_BIT_SGIX             = $00000020;
  GLX_STENCIL_BUFFER_BIT_SGIX           = $00000040;
  GLX_ACCUM_BUFFER_BIT_SGIX             = $00000080;
  GLX_SAMPLE_BUFFERS_BIT_SGIX           = $00000100;
  GLX_MAX_PBUFFER_WIDTH_SGIX            = $8016;
  GLX_MAX_PBUFFER_HEIGHT_SGIX           = $8017;
  GLX_MAX_PBUFFER_PIXELS_SGIX           = $8018;
  GLX_OPTIMAL_PBUFFER_WIDTH_SGIX        = $8019;
  GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX       = $801A;
  GLX_PRESERVED_CONTENTS_SGIX           = $801B;
  GLX_LARGEST_PBUFFER_SGIX              = $801C;
  GLX_WIDTH_SGIX                        = $801D;
  GLX_HEIGHT_SGIX                       = $801E;
  GLX_EVENT_MASK_SGIX                   = $801F;
  GLX_DAMAGED_SGIX                      = $8020;
  GLX_SAVED_SGIX                        = $8021;
  GLX_WINDOW_SGIX                       = $8022;
  GLX_PBUFFER_SGIX                      = $8023;

  //GLX_SGIX_video_resize
  GLX_SYNC_FRAME_SGIX                   = $00000000;
  GLX_SYNC_SWAP_SGIX                    = $00000001;

  //GLX_SGIX_dmbuffer
  GLX_DIGITAL_MEDIA_PBUFFER_SGIX        = $8024;

  //GLX_SGIS_blended_overlay
  GLX_BLENDED_RGBA_SGIS                 = $8025;

  //GLX_SGIS_shared_multisample
  GLX_MULTISAMPLE_SUB_RECT_WIDTH_SGIS   = $8026;
  GLX_MULTISAMPLE_SUB_RECT_HEIGHT_SGIS  = $8027;

  //GLX_3DFX_multisample
  GLX_SAMPLE_BUFFERS_3DFX               = $8050;
  GLX_SAMPLES_3DFX                      = $8051;

  //GLX_MESA_set_3dfx_mode
  GLX_3DFX_WINDOW_MODE_MESA             = $1;
  GLX_3DFX_FULLSCREEN_MODE_MESA         = $2;

  //GLX_SGIX_visual_select_group
  GLX_VISUAL_SELECT_GROUP_SGIX          = $8028;

  //iGLX_OML_swap_method
  GLX_SWAP_METHOD_OML                   = $8060;
  GLX_SWAP_EXCHANGE_OML                 = $8061;
  GLX_SWAP_COPY_OML                     = $8062;
  GLX_SWAP_UNDEFINED_OML                = $8063;

{ *********************************************************** }
type
  //GLX_ARB_get_proc_address

  //GLX_SGIX_video_source
  GLXVideoSourceSGIX                    = XID;

  //GLX_SGIX_fbconfig
  GLXFBConfigIDSGIX                     = XID;

  //GLX_SGIX_pbuffer
  GLXPbufferSGIX                        = XID;

var
  //GLX_SGI_swap_control
  glXSwapIntervalSGI : function (interval : glint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGI_video_sync
  glXGetVideoSyncSGI : function (count : pgluint): Pointer {PFNGLXGETVIDEOSYNCSGIPROC;} {$IFDEF LINUX} cdecl; {$ENDIF}
  glXWaitVideoSyncSGI : function (divisor : glint; remainder : glint; count : pgluint): Pointer {PFNGLXWAITVIDEOSYNCSGIPROC;} {$IFDEF LINUX} cdecl; {$ENDIF}

  glXMakeCurrentReadSGI : function (dpy : PDisplay; draw : GLXDrawable; read : GLXDrawable; ctx : GLXContext): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetCurrentReadDrawableSGI : function : GLXDrawable; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_video_source
  //glXCreateGLXVideoSourceSGIX : function (dpy : PDisplay; screen : glint; server : VLServer; path : VLPath; nodeClass : glint; drainNode : VLNode): GLXVideoSourceSGIX; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXDestroyGLXVideoSourceSGIX : procedure (dpy : PDisplay; glxvideosource : GLXVideoSourceSGIX); {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_fbconfig
  //glXGetFBConfigAttribSGIX : function (dpy : PDisplay; config : GLXFBConfigSGIX; attribute : glint; value : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXChooseFBConfigSGIX : function (dpy : PDisplay; screen : glint; attrib_list : pglint; nelements : pglint) PGLXFBConfigSGIX; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXCreateGLXPixmapWithConfigSGIX : function (dpy : PDisplay; config : GLXFBConfigSGIX; pixmap : Pixmap): GLXPixmap {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXCreateContextWithConfigSGIX : function (dpy : PDisplay; config : GLXFBConfigSGIX; render_type : glint; share_list : GLXContext; direct : glboolean): GLXContext; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXGetVisualFromFBConfigSGIX : function (dpy : PDisplay; config: GLXFBConfigSGIX): PXVisualInfo; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXGetFBConfigFromVisualSGIX : function (dpy : Display; cis : PXVisualInfo): GLXFBConfigSGIX {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_pbuffer
  //glXCreateGLXPbufferSGIX : function (dpy : PDisplay; config : GLXFBConfigSGIX; width : gluint; height : gluint; atrib_list : pglint): GLXPbufferSGIX; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXDestroyGLXPbufferSGIX : procedure (dpy : PDisplay; pbuf : GLXPbufferSGIX); {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXQueryGLXPbufferSGIX : function (dpy : Display; pbuf : GLXPbufferSGIX; attribute : glint; value : pgluint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXSelectEventSGIX : procedure (dpy : PDisplay; drawable : GLXDrawable; mask : gluint); {$IFDEF LINUX} cdecl; {$ENDIF}
  //glXGetSelectedEventSGIX : procedure (dpy : PDisplay; drawable : GLXDrawable; mask : pgluint); {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGI_cushion
  glXCushionSGI : procedure (dpy : PDisplay; window :  Window; cushion : glfloat); {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_video_resize
  glXBindChannelToWindowSGIX : function (dpy : PDisplay; screen : glint; channel : glint; window : Window): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXChannelRectSGIX : function (dpy : PDisplay; screen : glint; channel : glint; x : glint; y : glint; w : glint; h : glint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryChannelRectSGIX : function (dpy : PDisplay; screen : glint; channel : glint; dx : pglint; dy : pglint; dw : pglint; dh : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryChannelDeltasSGIX : function (dpy : PDisplay; screen : glint; channel : glint; x : pglint; y : pglint; w : pglint; g : pglint): glint; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXChannelRectSyncSGIX : function (dpy : PDisplay; screen : glint; channel : glint; synctype : GLenum): glint; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_dmbuffer
  //glXAssociateDMPbufferSGIX : function (dpy : PDisplay; pbuffer : GLXPbufferSGIX; params : PDMparams; dbbuffer : DMbuffer): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_swap_group
  glXJoinSwapGroupSGIX : procedure (dpy : PDisplay; drawable : GLXDrawable; member : GLXDrawable); {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SGIX_swap_barrier
  glXBindSwapBarrierSGIX : procedure (dpy : PDisplay; drawable : GLXDrawable; barrier : glint); {$IFDEF LINUX} cdecl; {$ENDIF}
  glXQueryMaxSwapBarriersSGIX : function (dpy : PDisplay; screen : glint; max : pglint): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_SUN_get_transparent_index
  glXGetTransparentIndexSUN : function (dpy : Display; overlay : Window; underlay : Window; pTransparentIndex : pglint): glint; //Status; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_MESA_copy_sub_buffer
  glXCopySubBufferMESA : procedure (dpy : PDisplay; drawable : GLXDrawable; x : glint; y : glint; width : glint; height : glint); {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_MESA_pixmap_colormap
  glXCreateGLXPixmapMESA : function (dpy : PDisplay; visual : PXVisualInfo; pixmap : Pixmap; cmap : Colormap): GLXPixmap; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_MESA_release_buffers
  glXReleaseBuffersMESA : function (dpy : PDisplay; drawable : GLXDrawable): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_MESA_set_3dfx_mode
  glXSet3DfxModeMESA : function (mode : glint): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_MESA_agp_offset
  glXGetAGPOffsetMESA : function (ptr : Pointer): GLuint; {$IFDEF LINUX} cdecl; {$ENDIF}

  //GLX_OML_sync_control
type
  pint64 = ^int64;

var
  glXGetSyncValuesOML : function (dpy : PDisplay; drawable : GLXDrawable; ust : pint64; msc : pint64; sbc : pint64): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXGetMscRateOML : function (dpy : PDisplay; drawable : GLXDrawable; numerator : pgluint; denominator : pgluint): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXSwapBuffersMscOML : function (dpy : PDisplay; drawabel : GLXDrawable; target_msc : int64;  divisor : int64; remainder : int64): int64; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXWaitForMscOML : function (dpy : PDisplay; drawable : GLXDrawable; target_msc : int64; divisor : int64; remainder : int64; ust : pint64; msc : pint64; sbc : pint64): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}
  glXWaitForSbcOML : function (dpy : PDisplay; drawable : GLXDrawable; target_sbc : int64; ust : pint64; msc : pint64; sbc : pint64): glboolean; {$IFDEF LINUX} cdecl; {$ENDIF}

{var
  GLX_VERSION_1_0,
  GLX_VERSION_1_1,
  GLX_VERSION_1_2,
  GLX_VERSION_1_3,
  GLX_VERSION_1_4,
  GLX_3DFX_multisample,
  GLX_ARB_get_proc_address,
  GLX_ARB_multisample,
  GLX_EXT_import_context,
  GLX_EXT_visual_info,
  GLX_EXT_visual_rating,
  GLX_MESA_agp_offset,
  GLX_MESA_copy_sub_buffer,
  GLX_MESA_pixmap_colormap,
  GLX_MESA_release_buffers,
  GLX_MESA_set_3dfx_mode,
  GLX_NV_vertex_array_range,
  GLX_OML_swap_method,
  GLX_OML_sync_control,
  GLX_SGIS_blended_overlay,
  GLX_SGIS_multisample,
  GLX_SGIS_shared_multisample,
  GLX_SGIX_dmbuffer,
  GLX_SGIX_fbconfig,
  GLX_SGIX_pbuffer,
  GLX_SGIX_swap_barrier,
  GLX_SGIX_swap_group,
  GLX_SGIX_video_resize,
  GLX_SGIX_video_source,
  GLX_SGIX_visual_select_group,
  GLX_SGI_cushion,
  GLX_SGI_make_current_read,
  GLX_SGI_swap_control,
  GLX_SGI_video_sync,
  GLX_SUN_get_transparent_index : Boolean; }

function InitGLX(LibHandle : THandle): Boolean;
procedure ClearGLX;
procedure ReadGLX;

implementation

var
  Handle : Cardinal;

function glxProcedure(ProcName : PChar): Pointer;
begin
  Result := nil;
  if Handle <> 0 then
  begin
    if Addr(glXGetProcAddress) <> NIL then
      Result := glXGetProcAddress(ProcName)
    else if Addr(glXGetProcAddressARB) <> NIL then
      Result := glXGetProcAddressARB(ProcName);

    if result = NIL then
      Result := GetProcAddress(Handle, ProcName);

    if ShowFunctions then
    begin
      if Result <> nil then
        WriteLn(Procname + ': 0x' + IntToHex(Cardinal(Result), 8))
      else
        WriteLn(Procname + ': off')
    end;
  end
end;


function InitGLX(LibHandle : THandle): Boolean;
begin
  Result := false;
  ClearGLX;
  Handle := LibHandle;
  if Handle <> 0 then
  begin
    glXGetProcAddress := GetProcAddress(Handle, 'glXGetProcAddress');
    glXGetProcAddressARB := GetProcAddress(Handle, 'glXGetProcAddressARB');
    ReadGLX;
    Result := true;
  end;
end;

procedure ClearGLX;
begin
  glXChooseVisual := nil;
  glXCopyContext := nil;
  glXCreateContext := nil;
  glXCreateGLXPixmap := nil;
  glXDestroyContext := nil;
  glXDestroyGLXPixmap := nil;
  glXGetConfig := nil;
  glXGetCurrentContext := nil;
  glXGetCurrentDrawable := nil;
  glXIsDirect := nil;
  glXMakeCurrent := nil;
  glXQueryExtension := nil;
  glXQueryVersion := nil;
  glXSwapBuffers := nil;
  glXUseXFont := nil;
  glXWaitGL := nil;
  glXWaitX := nil;

  glXGetClientString := nil;
  glXQueryServerString := nil;
  glXQueryExtensionsString := nil;

{  New for GLX 1.3  }
  glXGetFBConfigs := nil;
  glXChooseFBConfig := nil;
  glXGetFBConfigAttrib := nil;
  glXGetVisualFromFBConfig := nil;
  glXCreateWindow := nil;
  glXDestroyWindow := nil;
  glXCreatePixmap := nil;

  glXDestroyPixmap := nil;
  glXCreatePbuffer := nil;
  glXDestroyPbuffer := nil;
  glXQueryDrawable := nil;
  glXCreateNewContext := nil;
  glXMakeContextCurrent := nil;
  glXGetCurrentReadDrawable := nil;
  glXGetCurreentDisplay := nil;

  glXQueryContext := nil;
  glXSelectEvent := nil;
  glXGetSelectedEvent := nil;

{  GLX extensions  }
  glXGetContextIDEXT := nil;
  glXImportContextEXT := nil;
  glXFreeContextEXT := nil;
  glXQueryContextInfoEXT := nil;
  glXGetCurrentDisplayEXT := nil;

{ glxext.h }
  //GLX_SGI_swap_control
  glXSwapIntervalSGI := nil;

  //GLX_SGI_video_sync
  glXGetVideoSyncSGI := nil;
  glXWaitVideoSyncSGI := nil;

  glXMakeCurrentReadSGI := nil;
  glXGetCurrentReadDrawableSGI := nil;

  //GLX_SGIX_video_source
  //glXCreateGLXVideoSourceSGIX := nil;
  //glXDestroyGLXVideoSourceSGIX := nil;

  //GLX_SGIX_fbconfig
  //glXGetFBConfigAttribSGIX := nil;
  //glXChooseFBConfigSGIX := nil;
  //glXCreateGLXPixmapWithConfigSGIX := nil;
  //glXCreateContextWithConfigSGIX := nil;
  //glXGetVisualFromFBConfigSGIX := nil;
  //glXGetFBConfigFromVisualSGIX := nil;

  //GLX_SGIX_pbuffer
  //glXCreateGLXPbufferSGIX := nil;
  //glXDestroyGLXPbufferSGIX := nil;
  //glXQueryGLXPbufferSGIX := nil;
  //glXSelectEventSGIX := nil;
  //glXGetSelectedEventSGIX := nil;

  //GLX_SGI_cushion
  glXCushionSGI := nil;

  //GLX_SGIX_video_resize
  glXBindChannelToWindowSGIX := nil;
  glXChannelRectSGIX := nil;
  glXQueryChannelRectSGIX := nil;
  glXQueryChannelDeltasSGIX := nil;
  glXChannelRectSyncSGIX := nil;

  //GLX_SGIX_dmbuffer
  //glXAssociateDMPbufferSGIX := nil;

  //GLX_SGIX_swap_group
  glXJoinSwapGroupSGIX := nil;

  //GLX_SGIX_swap_barrier
  glXBindSwapBarrierSGIX := nil;
  glXQueryMaxSwapBarriersSGIX := nil;

  //GLX_SUN_get_transparent_index
  glXGetTransparentIndexSUN := nil;

  //GLX_MESA_copy_sub_buffer
  glXCopySubBufferMESA := nil;

  //GLX_MESA_pixmap_colormap
  glXCreateGLXPixmapMESA := nil;

  //GLX_MESA_release_buffers
  glXReleaseBuffersMESA := nil;

  //GLX_MESA_set_3dfx_mode
  glXSet3DfxModeMESA := nil;

  //GLX_MESA_agp_offset
  glXGetAGPOffsetMESA := nil;

  //GLX_OML_sync_control
  glXGetSyncValuesOML := nil;
  glXGetMscRateOML := nil;
  glXSwapBuffersMscOML := nil;
  glXWaitForMscOML := nil;
  glXWaitForSbcOML := nil;
end;

procedure ReadGLX;
begin
  glXChooseVisual := glxProcedure('glXChooseVisual');
  glXCopyContext := glxProcedure('glXCopyContext');
  glXCreateContext := glxProcedure('glXCreateContext');
  glXCreateGLXPixmap := glxProcedure('glXCreateGLXPixmap');
  glXDestroyContext := glxProcedure('glXDestroyContext');
  glXDestroyGLXPixmap := glxProcedure('glXDestroyGLXPixmap');
  glXGetConfig := glxProcedure('glXGetConfig');
  glXGetCurrentContext := glxProcedure('glXGetCurrentContext');
  glXGetCurrentDrawable := glxProcedure('glXGetCurrentDrawable');
  glXIsDirect := glxProcedure('glXIsDirect');
  glXMakeCurrent := glxProcedure('glXMakeCurrent');
  glXQueryExtension := glxProcedure('glXQueryExtension');
  glXQueryVersion := glxProcedure('glXQueryVersion');
  glXSwapBuffers := glxProcedure('glXSwapBuffers');
  glXUseXFont := glxProcedure('glXUseXFont');
  glXWaitGL := glxProcedure('glXWaitGL');
  glXWaitX := glxProcedure('glXWaitX');

  glXGetClientString := glxProcedure('glXGetClientString');
  glXQueryServerString := glxProcedure('glXQueryServerString');
  glXQueryExtensionsString := glxProcedure('glXQueryExtensionsString');

{  New for GLX 1.3  }
  glXGetFBConfigs := glxProcedure('glXGetFBConfigs');
  glXChooseFBConfig := glxProcedure('glXChooseFBConfig');
  glXGetFBConfigAttrib := glxProcedure('glXGetFBConfigAttrib');
  glXGetVisualFromFBConfig := glxProcedure('glXGetVisualFromFBConfig');
  glXCreateWindow := glxProcedure('glXCreateWindow');
  glXDestroyWindow := glxProcedure('glXDestroyWindow');
  glXCreatePixmap := glxProcedure('glXCreatePixmap');

  glXDestroyPixmap := glxProcedure('glXDestroyPixmap');
  glXCreatePbuffer := glxProcedure('glXCreatePbuffer');
  glXDestroyPbuffer := glxProcedure('glXDestroyPbuffer');
  glXQueryDrawable := glxProcedure('glXQueryDrawable');
  glXCreateNewContext := glxProcedure('glXCreateNewContext');
  glXMakeContextCurrent := glxProcedure('glXMakeContextCurrent');
  glXGetCurrentReadDrawable := glxProcedure('glXGetCurrentReadDrawable');
  glXGetCurreentDisplay := glxProcedure('glXGetCurreentDisplay');

  glXQueryContext := glxProcedure('glXQueryContext');
  glXSelectEvent := glxProcedure('glXSelectEvent');
  glXGetSelectedEvent := glxProcedure('glXGetSelectedEvent');

{  GLX extensions  }
  glXGetContextIDEXT := glxProcedure('glXGetContextIDEXT');
  glXImportContextEXT := glxProcedure('glXImportContextEXT');
  glXFreeContextEXT := glxProcedure('glXFreeContextEXT');
  glXQueryContextInfoEXT := glxProcedure('glXQueryContextInfoEXT');
  glXGetCurrentDisplayEXT := glxProcedure('glXGetCurrentDisplayEXT');

{ glxext.h }
  //GLX_SGI_swap_control
  glXSwapIntervalSGI := glxProcedure('glXSwapIntervalSGI');

  //GLX_SGI_video_sync
  glXGetVideoSyncSGI := glxProcedure('glXGetVideoSyncSGI');
  glXWaitVideoSyncSGI := glxProcedure('glXWaitVideoSyncSGI');

  glXMakeCurrentReadSGI := glxProcedure('glXMakeCurrentReadSGI');
  glXGetCurrentReadDrawableSGI := glxProcedure('glXGetCurrentReadDrawableSGI');

  //GLX_SGIX_video_source
  //glXCreateGLXVideoSourceSGIX := glxProcedure('glXCreateGLXVideoSourceSGIX');
  //glXDestroyGLXVideoSourceSGIX := glxProcedure('glXDestroyGLXVideoSourceSGIX');

  //GLX_SGIX_fbconfig
  //glXGetFBConfigAttribSGIX := glxProcedure('glXGetFBConfigAttribSGIX');
  //glXChooseFBConfigSGIX := glxProcedure('glXChooseFBConfigSGIX');
  //glXCreateGLXPixmapWithConfigSGIX := glxProcedure('glXCreateGLXPixmapWithConfigSGIX');
  //glXCreateContextWithConfigSGIX := glxProcedure('glXCreateContextWithConfigSGIX');
  //glXGetVisualFromFBConfigSGIX := glxProcedure('glXGetVisualFromFBConfigSGIX');
  //glXGetFBConfigFromVisualSGIX := glxProcedure('glXGetFBConfigFromVisualSGIX');

  //GLX_SGIX_pbuffer
  //glXCreateGLXPbufferSGIX := glxProcedure('glXCreateGLXPbufferSGIX');
  //glXDestroyGLXPbufferSGIX := glxProcedure('glXDestroyGLXPbufferSGIX');
  //glXQueryGLXPbufferSGIX := glxProcedure('glXQueryGLXPbufferSGIX');
  //glXSelectEventSGIX := glxProcedure('glXSelectEventSGIX');
  //glXGetSelectedEventSGIX := glxProcedure('glXGetSelectedEventSGIX');

  //GLX_SGI_cushion
  glXCushionSGI := glxProcedure('glXCushionSGI');

  //GLX_SGIX_video_resize
  glXBindChannelToWindowSGIX := glxProcedure('glXBindChannelToWindowSGIX');
  glXChannelRectSGIX := glxProcedure('glXChannelRectSGIX');
  glXQueryChannelRectSGIX := glxProcedure('glXQueryChannelRectSGIX');
  glXQueryChannelDeltasSGIX := glxProcedure('glXQueryChannelDeltasSGIX');
  glXChannelRectSyncSGIX := glxProcedure('glXChannelRectSyncSGIX');

  //GLX_SGIX_dmbuffer
  //glXAssociateDMPbufferSGIX := glxProcedure('glXAssociateDMPbufferSGIX');

  //GLX_SGIX_swap_group
  glXJoinSwapGroupSGIX := glxProcedure('glXJoinSwapGroupSGIX');

  //GLX_SGIX_swap_barrier
  glXBindSwapBarrierSGIX := glxProcedure('glXBindSwapBarrierSGIX');
  glXQueryMaxSwapBarriersSGIX := glxProcedure('glXQueryMaxSwapBarriersSGIX');

  //GLX_SUN_get_transparent_index
  glXGetTransparentIndexSUN := glxProcedure('glXGetTransparentIndexSUN');

  //GLX_MESA_copy_sub_buffer
  glXCopySubBufferMESA := glxProcedure('glXCopySubBufferMESA');

  //GLX_MESA_pixmap_colormap
  glXCreateGLXPixmapMESA := glxProcedure('glXCreateGLXPixmapMESA');

  //GLX_MESA_release_buffers
  glXReleaseBuffersMESA := glxProcedure('glXReleaseBuffersMESA');

  //GLX_MESA_set_3dfx_mode
  glXSet3DfxModeMESA := glxProcedure('glXSet3DfxModeMESA');

  //GLX_MESA_agp_offset
  glXGetAGPOffsetMESA := glxProcedure('glXGetAGPOffsetMESA');

  //GLX_OML_sync_control
  glXGetSyncValuesOML := glxProcedure('glXGetSyncValuesOML');
  glXGetMscRateOML := glxProcedure('glXGetMscRateOML');
  glXSwapBuffersMscOML := glxProcedure('glXSwapBuffersMscOML');
  glXWaitForMscOML := glxProcedure('glXWaitForMscOML');
  glXWaitForSbcOML := glxProcedure('glXWaitForSbcOML');
end;

end.
